package org.openxma.dsl.generator;

import java.util.List;
import java.util.Properties;

import org.openxma.dsl.common.DslProperties;
import org.openxma.dsl.core.model.ModelModifier;
import org.openxma.dsl.dom.model.NamingStrategy;
import org.openxma.dsl.dom.model.impl.DefaultRepositoryModelModifier;
import org.openxma.dsl.generator.impl.DefaultGeneratorContext;

import com.google.common.collect.Lists;

/**
 * Generator constants and provider that associates a
 * <code>GeneratorContext</code> instance with the current thread
 */
public class GeneratorProperties {
    /**
     * generator settings for outlets and property keys
     */
    public static final String BASE_DIR = "base.dir";
    public static final String OUTLET_JAVA = "outlet.java";
    public static final String OUTLET_TEST = "outlet.test";
    public static final String OUTLET_RESOURCES = "outlet.resources";
    public static final String AUTO_GENERATE = "auto.generate";
    public static final String OUTLET_GENERATED_JAVA = "outlet.generated.java";
    public static final String OUTLET_GENERATED_TEST = "outlet.generated.test";
    public static final String OUTLET_GENERATED_RESOURCES = "outlet.generated.resources";
    public static final String OUTLET_GENERATED_WSDL = "outlet.generated.wsdl";
    public static final String OUTLET_GENERATED_DOC = "outlet.generated.doc";
    public static final String XPAND_ASPECT_FILES = "templateAdviceFiles";
    public static final String PROJECT_WORKFLOW_FILES = "projectWorkflowFiles";
    public static final String DOMAIN_MODEL_GENERATOR_WORKFLOW_PATH = "workflow/DomGenerator.mwe";
    /**
     * the main Xpand template function
     */
    public static final String DOMAIN_MODEL_TEMPLATE_MAIN = "templates::Model::main";
    /**
     * the xsd/wsdl Xpand template functions
     */
    public static final String WSDL_TEMPLATE_MAIN = "templates::WSDL::main";
    public static final String XSD_TEMPLATE_MAIN = "templates::XSD::main";
    public static final String XSDINC_TEMPLATE_MAIN = "templates::XSDINC::main";
    /**
     * Xpand Outlet names
     */
    public static final String OUTLET_NAME_GENERATED_TEST = "OUTLET_GENERATED_TEST";
    public static final String OUTLET_NAME_TEST = "OUTLET_TEST";
    public static final String OUTLET_NAME_GENERATED_RESOURCES = "OUTLET_GENERATED_RESOURCES";
    public static final String OUTLET_NAME_RESOURCES = "OUTLET_RESOURCES";
    public static final String OUTLET_NAME_JAVA = "OUTLET_JAVA";
    public static final String OUTLET_NAME_WSDL = "OUTLET_GENERATED_WSDL";
    /**
     * Standard types
     */
    public static final List<String> DEFAULT_TYPES_LIST = Lists.newArrayList("BigDecimal", "Boolean", "Date", "Double",
            "Float", "Integer", "Long", "String", "Timestamp");
    /**
     * persistence related constants
     */
    public static final String DATABASE_VENDOR = "dataBase.vendor";
    public static final String DATABASE_VENDOR_DB2 = "DB2";
    public static final String DATABASE_VENDOR_H2 = "H2";
    public static final String DATABASE_VENDOR_HSQL = "HSQL";
    public static final String DATABASE_VENDOR_MYSQL5 = "MySQL5";
    public static final String DATABASE_VENDOR_SQLSERVER = "SQLSERVER";
    public static final String DATABASE_VENDOR_ORACLE10G = "ORACLE10G";
    
    /**
     * hibernate related constants
     */
    public static final String HIBERNATE_DIALECT_CLASS = "hibernate.dialect";

    public static final String HIBERNATE_USERTYPE_PROPERTY_POSTFIX = DefaultRepositoryModelModifier.HIBERNATE_USERTYPE_PROPERTY_POSTFIX;
    public static final String HIBERNATE_SQLTYPE_PROPERTY_POSTFIX = DefaultRepositoryModelModifier.HIBERNATE_SQLTYPE_PROPERTY_POSTFIX;
    public static final String HIBERNATE_USERTYPE_PROPERTY_FORMAT = DefaultRepositoryModelModifier.HIBERNATE_USERTYPE_PROPERTY_FORMAT;
    public static final String HIBERNATE_SQLTYPE_PROPERTY_FORMAT = DefaultRepositoryModelModifier.HIBERNATE_SQLTYPE_PROPERTY_FORMAT;


    private static final ThreadLocal<GeneratorContext> generatorContextHolder = new ThreadLocal<GeneratorContext>();

    /**
     * Reset the GeneratorContext for the current thread.
     */
    public static void resetGeneratorContext() {
        generatorContextHolder.set(null);
        DslProperties.Holder.resetDslProperties();
        ModelModifier.Holder.resetModelModifier();
        NamingStrategy.Holder.resetNamingStrategy();
    }

    /**
     * Associate the given GeneratorContext with the current thread, <i>not</i>
     * exposing it as inheritable for child threads.
     *
     * @param mapperContext
     *            the current GeneratorContext, or <code>null</code> to reset
     *            the thread-bound context
     */
    public static void setGeneratorContext(GeneratorContext mapperContext) {
        generatorContextHolder.set(mapperContext);
    }

    /**
     * Return the GeneratorContext associated with the current thread, if any.
     *
     * @return the current GeneratorContext
     */
    public static GeneratorContext getGeneratorContext() {
        GeneratorContext generatorContext = generatorContextHolder.get();
        if (generatorContext == null) {
            generatorContext = setProperties(null);
        }
        return generatorContext;
    }

    /**
     * Associate the given generator properties with the current thread.
     * <p>
     * Will implicitly create a GeneratorContext for the given properties.
     *
     * @param properties
     *            the generator <code>Properties</code> used by the generator,
     *            or <code>null</code> to reset the thread-bound context
     */
    public static GeneratorContext setProperties(Properties properties) {
        GeneratorContext generatorContext = new DefaultGeneratorContext(properties);
        setGeneratorContext(generatorContext);
        return generatorContext;
    }

    /**
     * @return the <code>ModelModifier</code> associated with the current
     *         thread, if any, or null else.
     */
    public static ModelModifier getModelModifier() {
        return getGeneratorContext().getModelModifier();
    }

    /**
     * @return the <code>LayoutStrategy</code> associated with the current
     *         thread, if any, or null else.
     */
    public static LayoutStrategy getLayoutStrategy() {
        return getGeneratorContext().getLayoutStrategy();
    }

    /**
     * @return the <code>NamingStrategy</code> associated with the current
     *         thread, if any, or null else.
     */
    public static NamingStrategy getNamingStrategy() {
        return getGeneratorContext().getNamingStrategy();
    }

    public static String getProperty(String key) {
        return getGeneratorContext().getProperty(key);
    }

    public static String getProperty(String key, String defaultValue) {
        return getGeneratorContext().getProperty(key, defaultValue);
    }

    public static boolean hasProperty(String key) {
        return null != getProperty(key);
    }
    /**
     * Reset the GeneratorContext for the current thread.
     */
    public static void reset() {
        resetGeneratorContext();
    }

    /**
     * @return the configured hibernate dialect class
     */
    public static String getHibernateDialect() {
        return getProperty(HIBERNATE_DIALECT_CLASS);
    }

}
