package org.openxma.dsl.generator;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.types.FileSet;

/**
 * Ant task for transforming one or more XMAdsl models to XMA.<br>
 * <h2>Usage:</h2>
 * <h3>Taskdef:</h3>
 * <pre>
 * &lt;taskdef name         = "name_you_like"
 *          classname    = "org.openxma.dsl.generator.XmaDslAntTask"
 *          classpathref = "classpath.id" /&gt;
 * </pre>
 * <h3>Classpath:</h3>
 * The classpath must contain<br>
 * <ul>
 *     <li>all the XMAdsl plugins (e.g. <code>org.openxma.dsl.*.jar</code>)</li>
 *  <li>the XMA guidesigner plugin (in reality we need the XMA model classes from there) (e.g. <code>at.spardat.xma.guidesigner*.jar</code>)</li>
 *  <li>the XMAdsl platform jar (e.g. <code>org.openxma.dsl.platform.*.jar</code>) - attention, this is not a plugin</li>
 *  <li>the project output directory, containing the XMAdsl file(s) (xText takes all from classpath)</li>
 *     <li>the EMF runtime jars (e.g. <code>org.eclipse.emf*.jar</code>)</li>
 *     <li>all the xText, xTend and antlr plugins (e.g. <code>org.eclipse.xtext*.jar:org.eclipse.xtend*.jar:org.antlr.runtime*.jar</code>)</li>
 *     <li>Google collect and guice plugins (e.g. <code>com.google.collect*.jar:com.google.guice&#42;/*&#42;/*.jar</code>)</li>
 *  <li>Apache cmd line tools plugin (e.g. <code>org.apache.commons.cli*.jar</code>)</li>
 *  <li>Apache Ant and Apache Commons plugins (e.g. <code>org.apache&#42;/*&#42;/*.jar:org.apache.commons.*.jar</code>)</li>
 *     <li>misc Eclipse core stuff (<code>org.eclipse.core.jobs*.jar:org.eclipse.core.resources*.jar:org.eclipse.core.runtime*.jar:org.eclipse.osgi*.jar:org.eclipse.osgi*.jar:org.eclipse.equinox.common*.jar:org.eclipse.swt*.jar:com.ibm.icu*.jar</code>)<br></li>
 * </ul>
 *
 * <h3>Usage with one ui model file:</h3>
 * <pre>
 * &lt;xmadsl file             = "C:/myworkspace/myproject/src/org/openxma/dsl/example/simple/Example.xmadsl"
 *         platformUri      = "C:/myworkspace"&gt;
 *     &lt;classpath&gt;
 *         &lt;path refid = "classpath.id" /&gt;
 *     &lt;/classpath&gt;
 * &lt;/xmadsl&gt;
 * </pre>
 *
 * <h3>Usage with a domain model folder (simple project layout):</h3>
 * <pre>
 * &lt;xmadsl platformUri = "C:/myworkspace"
 *            domSourceFolder = "src/models" &gt;
 *     &lt;classpath&gt;
 *         &lt;path refid = "classpath.id" /&gt;
 *     &lt;/classpath&gt;
 * &lt;/xmadsl&gt;
 * </pre>
 *
 * <h3>Usage with a ui model file set and a domain model folder with all possible settings (complex maven project layout):</h3>
 * <pre>
 * &lt;xmadsl platformUri             = "C:/myworkspace"
 *         domSourceFolder         = "src/model/java"
 *         javaSourceFolder        = "src/main/java"
 *         javaGenSourceFolder     = "src/generated/java"
 *         resourceSourceFolder    = "src/main/resources"
 *         resourceGenSourceFolder = "src/generated/resources"
 *         cleanDomGenDirectory    = "true"
 *         templateAdviceFiles     = "templatefiel1.xxy,template2.yyx"
 * &gt;
 *     &lt;classpath&gt;
 *         &lt;path refid = "classpath.id" /&gt;
 *     &lt;/classpath&gt;
 *     &lt;fileset dir = "C:/myworkspace/myproject/src"&gt;
 *         ...
 *     &lt;/fileset&gt;
 * &lt;/xmadsl&gt;
 * </pre>
 *
 * @author laslovd
 *
 */
public class XmaDslAntTask extends WorkflowAntTask {

    private final static String DEFAULT_SRC_FOLDER = "SRC";

    private final Map<String,Object> paramMap = new HashMap<String,Object>();
    private final ArrayList<FileSet> fileSets = new ArrayList<FileSet>();
    private String platformUri;
    private String javaSourceFolder = DEFAULT_SRC_FOLDER;
    private String javaGenSourceFolder = javaSourceFolder;
    private String resourceSourceFolder = javaSourceFolder;
    private String resourceGenSourceFolder = javaSourceFolder;
    private String domSourceFolder;
    private String templateAdviceFiles;
    private String extensionAdviceFiles;
    private boolean cleanDomGenDirectory = false;
    private File file;
    private File projectDir;


    /**
     * Source folder for all the domain model files
     * @param domSourceFolder
     */
    public void setDomSourceFolder(String domSourceFolder) {
        this.domSourceFolder = domSourceFolder;
    }

    /**
     * the project java source folder
     * (also destination for the generated xma files) <br>
     * optional, default = "src"
     */
    public void setJavaSourceFolder(String javaSourceFolder) {
        this.javaSourceFolder = javaSourceFolder;
    }

    /**
     * the project java source folder for generated stuff<br>
     * optional, default = same than "javaSourceFolder"
     */
    public void setJavaGenSourceFolder(String javaGenSourceFolder) {
        this.javaGenSourceFolder = javaGenSourceFolder;
    }

    /**
     * the project resource source folder<br>
     * optional, default = "src"
     */
    public void setResourceSourceFolder(String resourceSourceFolder) {
        this.resourceSourceFolder = resourceSourceFolder;
    }

    /**
     * the project resource source folder for generated stuff<br>
     * optional, default = same than resourceSourceFolder
     */
    public void setResourceGenSourceFolder(String resourceGenSourceFolder) {
        this.resourceGenSourceFolder = resourceGenSourceFolder;
    }

    /**
     * switch for cleaning first the dom domain generated stuff
     */
    public void setCleanDomGenDirectory(boolean cleanDomGenDirectory) {
        this.cleanDomGenDirectory = cleanDomGenDirectory;
    }

    /**
     * One or more (comma seperated) template advice files.<br>
     * @see http://snapshots.dist.codehaus.org/openxma/docs/XMAdsl/DomainModelGenerator.html#generator.Workflow
     * @param templateAdviceFiles
     */
    public void setTemplateAdviceFiles(String templateAdviceFiles) {
        this.templateAdviceFiles = templateAdviceFiles;
    }

    // Not used yet, for further development
//    public void setExtensionAdviceFiles(String extensionAdviceFiles) {
//        this.extensionAdviceFiles = extensionAdviceFiles;
//    }

    /**
     * Needed by the workflow runner for standalone (outside of Eclipse) execution
     * @param platformUri The Eclipse workspace (mandatory)
     * @throws IOException
     */
    public void setPlatformUri ( String platformUri ) throws IOException {
        File platformFile = new File(platformUri);
        this.platformUri = toUniversalSlashes(platformFile.getCanonicalPath());
    }

    /**
     * Not longer needed
     * @param projectName
     */
    public void setProjectName ( String projectName ) {
        sayAttrIsDepricated("projectName");
    }

    /**
     * Not longer needed
     * @param classPathUri
     */
    public void setClassPathUri(String classPathUri) {
        sayAttrIsDepricated("classPathUri");
    }

    /**
     * Not longer needed
     * @param projectDir
     */
    public void setProjectDir ( File projectDir ) {
        sayAttrIsDepricated("projectDir");
    }

    /**
     * For one file execution
     * @param file
     */
    public void setFile ( File file ) {
        this.file = file;
    }

    /**
     * For multiple file execution
     * @param fileset
     */
    public void addFileset ( FileSet fileset ) {
        fileSets.add(fileset);
    }

    /**
     * Execution, called by Ant. Calls (indirectly) the superclass implementation, possible several times.
     * @see WorkflowAntTask#execute()
     */
    @Override
    public void execute() throws BuildException {
        if ( javaGenSourceFolder == DEFAULT_SRC_FOLDER && javaSourceFolder != DEFAULT_SRC_FOLDER )
            javaGenSourceFolder = javaSourceFolder;
        if ( resourceGenSourceFolder == DEFAULT_SRC_FOLDER && resourceSourceFolder != DEFAULT_SRC_FOLDER )
            resourceGenSourceFolder = resourceSourceFolder;
        check();

        // dom stuff:
        if (domSourceFolder != null) {
            super.setFile("workflow/DomGenerator.mwe");
            setParam("outlet.java", mkAbsoluteDir(javaSourceFolder));
            String or = mkAbsoluteDir(resourceSourceFolder);
            setParam("outlet.resources", or);
            setParam("outlet.generated.java", mkAbsoluteDir(javaGenSourceFolder));
            String ogr = mkAbsoluteDir(resourceGenSourceFolder);
            setParam("outlet.generated.resources", ogr);
            setParam("modelFolder", mkAbsoluteDir(domSourceFolder));
            if (templateAdviceFiles != null)
                setParam("templateAdviceFiles", templateAdviceFiles);
            if (extensionAdviceFiles != null)
                setParam("extensionAdviceFiles", extensionAdviceFiles);
            setParam("cleanDirectory", new Boolean(cleanDomGenDirectory).toString());

            //start: for hbm file merge
            Properties props = new Properties();
            props.put(GeneratorProperties.OUTLET_RESOURCES, or);
            props.put(GeneratorProperties.OUTLET_GENERATED_RESOURCES, ogr);
            GeneratorProperties.setProperties(props);
            //end: for hbm file merge

            super.execute();
        }

        // ui stuff:

        // set first the things that stays the same for all files:
        setParam("platformUri",platformUri);
        setParam("javaSourceFolder",javaSourceFolder);
        super.setFile("workflow/Pom2Xma.mwe");

        // get all files
        Set<File> files = new TreeSet<File>();
        // one file got?
        if ( file != null ) {
            files.add(file);
        }
        // one or more filesets got?
        for ( Iterator<FileSet> itFSets = fileSets.iterator(); itFSets.hasNext(); ) {
            FileSet fs = itFSets.next();
            DirectoryScanner ds = fs.getDirectoryScanner(getProject());
            File base = ds.getBasedir();
            String[] includedFiles = ds.getIncludedFiles();
            for ( int i = 0; i < includedFiles.length; i++ ) {
                File file = new File(base,includedFiles[i]);
                files.add(file);
            }
        }

        for ( File file : files ) {
            try {
                executeOneUiFile(file);
            } catch (IOException e) {
                throw new BuildException(e);
            }
        }

        log(files.size() + " ui model files transformed.");
    }

    // checks existence of necessary params
    private void check () throws BuildException {
        if ( platformUri == null || platformUri.length() < 1 ) {
            throw new BuildException("Missing attribute 'platformUri'");
        }
        if ( javaSourceFolder == null ) {
            throw new BuildException("Missing attribute 'javaSourceFolder'");
        }
        if ( domSourceFolder == null && file == null && fileSets.size() == 0 ) {
            throw new BuildException("No ui model files and no domain model folder specified. For ui model files User either the 'file' attribute or a 'fileset' element. For the domain model folder use the 'domSourceFolder' attribute.");
        }
    }

    // extracts the two needed, file related parameters, sets them as used by superclass and
    // calls superclass execute() method
    private void executeOneUiFile ( File file ) throws BuildException, IOException {
        // sample, what we have:
        // platformUri      = "C:/fleXMA/workspace/"
        // javaSourceFolder = "/src"
        // file             = "c:\fleXMA\workspace\test\src\org\openxma\dsl\example\simple\Example.xmadsl"
        // ==========================================================
        // then we would need to get out:
        // componentFolder = src/org/openxma/dsl/example/simple
        // componentName   = Example
        // classPathUri    = classpath:/org/openxma/dsl/example/simple/Example.xmadsl
        // projectName     = test

        File platformFile   = new File(platformUri);
        String platformAS   = platformFile.getAbsolutePath();
        String fileAS       = file.getCanonicalPath();
        String delta        = fileAS.substring(platformAS.length() + 1); // "test\src\org\openxma\dsl\example\simple\Example.xmadsl"
        String projectName  = delta.substring(0,delta.indexOf(File.separatorChar)); // "test"
        setParam("projectName",projectName);

        delta               = delta.substring(delta.indexOf(File.separatorChar) + 1);
        delta               = delta.substring(0,delta.lastIndexOf(File.separatorChar));
        String compFolder   = toUniversalSlashes(delta);
        setParam("componentFolder",compFolder);

        File projectFile    = new File(platformFile,projectName);
        File srcFile        = new File(projectFile,javaSourceFolder);
        String srcAS        = srcFile.getCanonicalPath();
        delta               = fileAS.substring(srcAS.length() + 1); // "org\openxma\dsl\example\simple\Example.xmadsl"
        String classPathUri = toUniversalSlashes(delta);
        setParam("classPathUri","classpath:/" + classPathUri);

        String fileName     = file.getName();
        String compName     = fileName.substring(0,fileName.lastIndexOf('.'));
        setParam("componentName",compName);

        super.execute();
    }

    /**
     * sets parameter values. if not used before,
     * parameter will be created by superclass and stored in a subclass map to can reuse it
     */
    private void setParam ( String name, String value ) throws BuildException {
        try {
            log("setParam: " + name + " = '" + value + "'", Project.MSG_VERBOSE);
            Object p = paramMap.get(name);
            if ( p == null ) {
                p = super.createParam();
                // Bloody OAW/xText guys! They changed the Parameter class from the normal Ant
                // stuff to something internal that is not accessible! So we have to use reflection now! Argh!
                Method method =  p.getClass().getMethod("setName",new Class[] { String.class } );
                method.invoke(p,new Object[] { name });
                paramMap.put(name,p);
            }
            Method method =  p.getClass().getMethod("setValue",new Class[] { String.class } );
            method.invoke(p,new Object[] { value });
        }
        catch ( Exception ex ) {
            throw new BuildException("Setting of parameter '" + name + "' failed!",ex);
        }
    }

    private void sayAttrIsDepricated ( String attr ) {
        log("Setting attribute '"  + attr + "' is depricated and has no effect. "
                           + "The setting will now be extracted from the file attribute or the fileset members.", Project.MSG_WARN);
    }

    private String mkAbsoluteDir ( final String folder ) {
        File f = new File(folder).getAbsoluteFile();
        if ( f.isDirectory() ) // is absolute or relative to where we are?
            return toUniversalSlashes(f.getAbsolutePath());

        if ( projectDir == null ) {
            File platformFile = new File(platformUri);
            File currentDir   = new File(".");
            String platformString = platformFile.getAbsolutePath();
            String currentDirString = toUniversalSlashes(currentDir.getAbsolutePath());
            int p = currentDirString.indexOf('/',platformString.length() + 1);
            if ( p < 0 )
                throw new BuildException("Cannot retrieve project dir from current dir!");
            projectDir = new File(currentDirString.substring(0,p));
        }

        f = new File(projectDir,folder);
        if ( f.isDirectory() ) // is project relative?
            return toUniversalSlashes(f.getAbsolutePath());

        throw new BuildException("Cannot find a directory matching '" + folder + "'!");
    }

    private String toUniversalSlashes ( String pathOrFile ) {
        return pathOrFile.replace('\\','/');
    }
}
