package org.openxma.dsl.generator.component;

import java.io.IOException;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.mwe.core.WorkflowContext;
import org.eclipse.emf.mwe.core.WorkflowInterruptedException;
import org.eclipse.emf.mwe.core.issues.Issues;
import org.eclipse.emf.mwe.core.monitor.ProgressMonitor;
import org.eclipse.xtext.mwe.Reader;
import org.eclipse.xtext.mwe.Validator;
import org.eclipse.xtext.resource.IResourceServiceProvider;
import org.eclipse.xtext.resource.IResourceServiceProvider.Registry;
import org.eclipse.xtext.validation.CheckMode;
import org.eclipse.xtext.validation.Issue;
import org.openxma.dsl.common.DslConstants;
import org.openxma.dsl.pom.PomDslStandaloneSetup;

import com.google.common.collect.Lists;
import com.google.inject.Injector;

public class PmlReader extends Reader {
	String pmlUri;

	public void setPmlUri(String pmlUri) {
		this.pmlUri = pmlUri;
	}

	@Override
	protected void checkConfigurationInternal(Issues issues) {
		addRegister(new PomDslStandaloneSetup());
		setUseJavaClassPath(true);
		if (pmlUri == null) {
			issues.addError(this, "No pml uri configured (property 'pmlUri')");
		}
	}

	@Override
	protected void addModelElementsToContext(WorkflowContext ctx, ResourceSet resourceSet) {
		EList<Resource> resources = resourceSet.getResources();
		for (Resource resource : resources) {
			URI normalizedPmlUri = resourceSet.getURIConverter().normalize(URI.createURI(pmlUri));
			URI resourceUri = resourceSet.getURIConverter().normalize(resource.getURI());
			if (normalizedPmlUri.toFileString().equals(resourceUri.toFileString())) {
				ctx.set("model", resource.getEObject("/"));
				return;
			}
		}
	}

	@Override
	protected void invokeInternal(WorkflowContext ctx, ProgressMonitor monitor, Issues issues) {
		Injector injector = getInjectors().iterator().next();
		IResourceServiceProvider xmaServiceProvider = injector
				.getInstance(org.eclipse.xtext.resource.IResourceServiceProvider.class);
		IResourceServiceProvider.Registry.INSTANCE.getExtensionToFactoryMap().put(DslConstants.OPENXMA_XMA_FILE_EXTENSION,
				xmaServiceProvider);
		super.invokeInternal(ctx, monitor, issues);
	}

	@Override
	protected Validator getValidator() {
		return new Validator() {
			@Override
			public void validate(ResourceSet resourceSet, Registry registry, Issues issues) {
				List<Resource> resources = Lists.newArrayList(resourceSet.getResources());
				for (Resource resource : resources) {
					if (resource.getURI().fileExtension().equals(DslConstants.OPENXMA_XMA_FILE_EXTENSION)) {
						continue;
					}
					try {
						resource.load(null);
						IResourceServiceProvider provider = registry.getResourceServiceProvider(resource.getURI());
						if (provider != null) {
							List<Issue> result = provider.getResourceValidator().validate(resource, CheckMode.ALL, null);
							for (Issue issue : result) {
								switch (issue.getSeverity()) {
								case ERROR:
									issues.addError(issue.getMessage(), issue);
									break;
								case WARNING:
									issues.addWarning(issue.getMessage(), issue);
									break;
								case INFO:
									issues.addInfo(issue.getMessage(), issue);
									break;
								}
							}
						}
					} catch (IOException e) {
						throw new WorkflowInterruptedException("Couldn't load resource (" + resource.getURI() + ")", e);
					}
				}
				if (isStopOnError() && issues.hasErrors()) {
					String errorMessage = toString(issues);
					throw new WorkflowInterruptedException("Validation problems: \n" + errorMessage);
				}
			}
		};
	}

}
