package org.openxma.dsl.generator.helper;

import static org.openxma.dsl.generator.helper.FileUtil.getPackagedFileName;

import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.xtext.EcoreUtil2;
import org.openxma.dsl.core.model.Call;
import org.openxma.dsl.core.model.Function;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.core.model.ModelElement;
import org.openxma.dsl.core.model.ReferenceableByXmadslVariable;
import org.openxma.dsl.core.model.StatusFlag;
import org.openxma.dsl.core.model.XmadslVariable;
import org.openxma.dsl.dom.model.ApplicationSession;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.AttributeProperty;
import org.openxma.dsl.dom.model.AttributeTextProperty;
import org.openxma.dsl.dom.model.Column;
import org.openxma.dsl.dom.model.Dao;
import org.openxma.dsl.dom.model.DataView;
import org.openxma.dsl.dom.model.Entity;
import org.openxma.dsl.dom.model.ManyToMany;
import org.openxma.dsl.dom.model.NamingStrategy;
import org.openxma.dsl.dom.model.Property;
import org.openxma.dsl.dom.model.Service;
import org.openxma.dsl.generator.GeneratorProperties;

/**
 * Represents an extension class invoked from within xtend files intended to be
 * used for entity related functions.
 *
 */
public class EntityExtension {

    public static void modify(Model model) {
        GeneratorProperties.getModelModifier().modifyModel(model);
    }

    /**
     * @return the join column name of the opposite many-to-many mapping
     */
    public static String getJoinColumnName(ManyToMany manyToMany) {
        Entity referencedEntity = (Entity) manyToMany.getAttribute().getDataType();
        Attribute opposite = getOpposite(manyToMany);
        if (opposite != null) {
            ManyToMany oppositeManyToMany = referencedEntity.getRepository().getManyToMany(opposite);
            if (oppositeManyToMany.getColumnName() != null) {
                return oppositeManyToMany.getColumnName();
            }
        }
        Column identifierColumn = referencedEntity.getRepository().getColumnDefinition(
                (Attribute) referencedEntity.getIdentifier());
        return identifierColumn != null ? identifierColumn.getColumnName() : getColumnName(referencedEntity,
                (Attribute) referencedEntity.getIdentifier());
    }

    private static Attribute getOpposite(ManyToMany manyToMany) {
        Attribute opposite = manyToMany.getAttribute().getOpposite();
        if (opposite == null) {
            Entity entity = (Entity) manyToMany.getAttribute().getDataType();
            EList<Attribute> attributes = entity.getAttributes();
            for (Attribute attribute : attributes) {
                if (attribute.getOpposite() != null && attribute.getOpposite().equals(manyToMany.getAttribute())) {
                    opposite = attribute;
                    break;
                }
            }
        }
        return opposite;
    }

    /**
     * @see NamingStrategy#getColumnName(Entity,Attribute)
     */
    public static String getColumnName(Entity entity, Attribute attribute) {
        return GeneratorProperties.getNamingStrategy().getColumnName(entity, attribute);
    }

    /**
     * @see NamingStrategy#getPackageName(Service)
     */
    public static String getPackageName(ModelElement eObject) {
        return GeneratorProperties.getNamingStrategy().getPackageName(eObject);
    }

    /**
     * @see NamingStrategy#getInterfacePackageName(Service)
     */
    public static String getInterfacePackageName(ModelElement eObject) {
        return GeneratorProperties.getNamingStrategy().getInterfacePackageName(eObject);
    }

    public static String getGeneratedHbmXmlFileName(Dao dao) {
        // the suffix ensures, that hibernate will ignore such files
        return getPackagedFileName(getPackageName(dao), dao.getEntity().getName(), "hbm.gen.xml");
    }

    public static String getFragmentHbmXmlFileName(Dao dao) {
        // the suffix ensures, that hibernate will ignore such files
        return getPackagedFileName(getPackageName(dao), dao.getEntity().getName(), "hbm.fragment.xml");
    }

    public static String getHbmXmlFileName(Dao dao) {
        // the suffix ensures, that hibernate will ignore such files
        return getPackagedFileName(getPackageName(dao), dao.getEntity().getName(), "hbm.xml");
    }

    /**
     * @see NamingStrategy#getDisplayPropertiesFile(Model)
     */
    public static String getDisplayPropertiesFile(Model model) {
        return getPackagedFileName(model.getName(), "display", "properties");
    }

    /**
     *
     * @param model
     * @return the package name for the given DomainModel
     */
    public static String getPackageName(Model model) {
        return (model != null) ? model.getName() : null;
    }

    /**
     * @param dataView
     * @param entity
     * @return true or false whether the dataview contains all required features
     *         for the given entity
     */
    public static boolean includesAllRequiredFeaturesFor(DataView dataView, Entity entity) {
        return dataView.includesAllRequiredFeaturesFor(entity);
    }

    /**
     * @param dataView
     * @param entity
     * @return true or false whether the dataview included the version attribute
     *         for the given entity
     */
    public static boolean includesVersionFor(DataView dataView, Entity entity) {
        return dataView.includesVersionFor(entity);
    }

    /**
     * @param attribute
     * @return true or false whether the given attribute represents the inverse
     *         side in an bi-directional relation
     */
    public static boolean isInverseSide(Attribute reference) {
        Entity entity = (Entity) reference.eContainer();
        Dao repository = entity.getRepository();
        ManyToMany manyToMany = repository != null ? repository.getManyToMany(reference) : null;
        if (manyToMany != null) {
            return manyToMany.isInverse();
        }
        return reference.getOpposite() != null && !reference.getOpposite().isMany();
    }

    /**
     * @param entity
     * @return true or false whether the given entity contains derived
     *         attributes
     */
    public static boolean containsDerivedAttributes(Entity entity, boolean localAttributesOnly) {
        Collection<Attribute> cloneAttributes = localAttributesOnly ? EcoreUtil.copyAll(entity.getAttributes())
                : entity.copyAttributes();
        for (Attribute attribute : cloneAttributes) {
            if (attribute.isDerived()) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param entity
     * @return the required application context referenced from within
     *         expressions declared within the given entity
     */
    public static ApplicationSession getRequiredApplicationContext(Entity entity) {
        List<XmadslVariable> variables = EcoreUtil2.getAllContentsOfType(entity, XmadslVariable.class);
        for (XmadslVariable xmadslVariable : variables) {
            ReferenceableByXmadslVariable reference = xmadslVariable.getReference();
            if (reference instanceof Property && reference.eContainer() instanceof ApplicationSession) {
                return (ApplicationSession) reference.eContainer();
            } else if (reference instanceof StatusFlag
                    && reference.eContainer().eContainer() instanceof ApplicationSession) {
                return (ApplicationSession) reference.eContainer().eContainer();
            }
        }

        List<Call> calls = EcoreUtil2.getAllContentsOfType(entity, Call.class);

        if (!calls.isEmpty()) {
            Function function = calls.iterator().next().getFunction();
            if (function.eContainer() instanceof ApplicationSession) {
                return (ApplicationSession) function.eContainer();
            }
        }
        return null;
    }
    
    /**
     * @return the description text of the given attribute
     */
    public static String getComment(Column column) {
        EList<AttributeProperty> attributProperties = column.getAttribute().getAttributProperties();
        if (attributProperties != null) {
            for (AttributeProperty attributProperty: attributProperties) {
                if (attributProperty instanceof AttributeTextProperty && ((AttributeTextProperty)attributProperty).getTooltipText() != null) {
                    return ((AttributeTextProperty)attributProperty).getTooltipText();
                }
            }
        }
        return null;
    }
}
