package org.openxma.dsl.generator.helper;

import static org.openxma.dsl.generator.GeneratorProperties.getProperty;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtend.expression.ExecutionContext;
import org.eclipse.xtend.expression.IExecutionContextAware;
import org.openxma.dsl.common.DslConstants;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.core.model.ModelElement;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.core.model.SqlType;
import org.openxma.dsl.core.model.Type;
import org.openxma.dsl.dom.model.ApplicationSession;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.ComplexType;
import org.openxma.dsl.dom.model.Dao;
import org.openxma.dsl.dom.model.DataView;
import org.openxma.dsl.dom.model.Entity;
import org.openxma.dsl.dom.model.FeatureReference;
import org.openxma.dsl.dom.model.Property;
import org.openxma.dsl.dom.model.Service;
import org.openxma.dsl.generator.component.JavaImportPostProcessor;

/**
 * Internal naming related utilities, used by templates and transformations.
 */
public class NameExtension implements IExecutionContextAware {

	/**
	 * Determine the package of a URI
	 * 
	 * @param uri
	 * @return
	 */
	public static String getPackageOfResource(URI uri) {
		String uriString = uri.toString();
		if (uriString.indexOf('/') > 0) {
			return uriString.substring(null != uri.scheme() ? uri.scheme().length() + 2 : 0, uriString.lastIndexOf('/'))
					.replace('/', '.');
		} else {
			return null;
		}
	}

	/**
	 * 
	 * @param complexType
	 * @return
	 */
	public static String getFullClassNameForDatamapping(ComplexType complexType) {
		String packageName = ((Model) complexType.eContainer()).getName();
		if (complexType instanceof DataView || complexType instanceof Entity) {
			packageName += "." + getProperty(DslConstants.DOMAIN_MODEL_DTO_PACKAGE);
		}
		return packageName + "." + getDatamappingClassName(complexType);
	}

	/**
	 * @deprecated use {@link #getDefaultDataViewName(Entity)} isntead
	 */
	public static String getDatamappingClassName(ComplexType complexType) {
		if (complexType instanceof Entity) {
			return complexType.getName() + "View";
		}
		return complexType.getName();
	}

	public static String getPackageName(EObject eObject, Boolean implementation) {
		return NameExtension.getPackageNameInternal(eObject, implementation);
	}

	public static String getPackageNameInternal(EObject eObject, Boolean implementation) {
		if (eObject instanceof ModelElement) {
			ModelElement modelElement = (ModelElement) eObject;
			if (eObject instanceof DataView || eObject instanceof SimpleType) {
				return EntityExtension.getPackageName(modelElement);
			}
			if (eObject instanceof Entity || eObject instanceof Service || eObject instanceof Dao
					|| eObject instanceof ApplicationSession) {
				if (implementation.booleanValue()) {
					return EntityExtension.getPackageName(modelElement);
				} else {
					return EntityExtension.getInterfacePackageName(modelElement);
				}
			}
			return ((Model) modelElement.eContainer()).getName();
		}
		throw new IllegalStateException("Method getPackageName() not defined for EObject " + eObject);
	}

	private String getFullyQualifiedName(SimpleType simpleType, Boolean implementation) {		
		if (simpleType.isCustom() && simpleType.getTypeDefinition().getDefinition() != null) {
			return getPackageNameInternal(simpleType, implementation) + "." + getName(simpleType, implementation);
		}
		return simpleType.getInstanceTypeName();
	}

	private String getFullyQualifiedName(SqlType sqlType, Boolean implementation) {
		if (sqlType.getDdl() != null) {
			return sqlType.getDdl();
		}
		return sqlType.getName();
	}

	private String getFullyQualifiedName(Type type, Boolean implementation) {
		if (type instanceof SimpleType) {
			return getFullyQualifiedName((SimpleType) type, implementation);
		} else if (type instanceof SqlType) {
			return getFullyQualifiedName((SqlType) type, implementation);
		} else {
			if (type.eContainer() instanceof Model || ((Model)type.eContainer()).getName()!=null) {
				return getPackageNameInternal(type, implementation) + "." + getName(type, implementation);
			}
			return getName(type, implementation);
		}
	}

	public static String getFullyQualifiedName(EObject eObject, Boolean implementation) {
		if (eObject instanceof Service) {
			return getPackageName(eObject, implementation) + "." + getName(eObject, implementation);
		}
		if (eObject instanceof Type) {
			return new NameExtension().getFullyQualifiedName(((Type) eObject), implementation);
		}
		if (eObject instanceof DataTypeAndTypeParameter) {
			return new NameExtension().getFullyQualifiedName(((DataTypeAndTypeParameter) eObject).getDataType(),
					implementation);
		}
		if (eObject instanceof Dao) {
			return getPackageName(eObject, implementation) + "." + getName(eObject, implementation);
		}
		if (eObject instanceof ModelElement) {
			if (eObject.eContainer() instanceof ModelElement) {
				return getPackageName(eObject, implementation) + "." + getName(eObject.eContainer(), implementation);
			}
			return getName(eObject.eContainer(), implementation);
		}
		throw new IllegalStateException("Method getFullyQualifiedName() not defined for EObject " + eObject);
	}

	private static String getNameInternal(EObject eObject, Boolean implementation) {
		if (eObject instanceof Attribute) {
			return ((Attribute) eObject).getName();
		}
		if (eObject instanceof Property) {
			return ((Property) eObject).getName();
		}
		if (eObject instanceof FeatureReference) {
			FeatureReference featureReference = (FeatureReference) eObject;
			return featureReference.getName()!=null ? featureReference.getName() : featureReference.getAttribute().getName();
		}
		if (eObject instanceof ModelElement) {
			ModelElement modelElement = (ModelElement) eObject;
			return modelElement.getName();
		}
		throw new IllegalStateException("Method getName() not defined for EObject " + eObject);
	}

	public static String getName(EObject eObject, Boolean implementation) {
		return NameExtension.getNameInternal(eObject, implementation);
	}

	public static String getName(EObject eObject) {
		return NameExtension.getNameInternal(eObject, false);
	}

	public String getFullyQualifiedNameAndRegister(EObject eObject, Boolean implementation) {
		return this.importPostProcessor.registerImport(getFullyQualifiedName(eObject, implementation));
	}

	public void setExecutionContext(ExecutionContext ctx) {
		this.importPostProcessor = (JavaImportPostProcessor) ctx.getGlobalVariables().get("__javaImportTool").getValue();
	}

	private JavaImportPostProcessor importPostProcessor;
}
