package org.openxma.dsl.generator.impl;

import java.util.Properties;

import org.eclipse.emf.mwe.core.resources.ResourceLoaderFactory;
import org.openxma.dsl.common.DslConstants;
import org.openxma.dsl.common.DslProperties;
import org.openxma.dsl.core.model.ModelModifier;
import org.openxma.dsl.dom.model.NamingStrategy;
import org.openxma.dsl.generator.GeneratorContext;
import org.openxma.dsl.generator.LayoutStrategy;

/**
 * Default implementation of the {@link GeneratorContext} interface which uses a
 * simple property override mechanism for determining the final set of property
 * values. <br/>
 * Properties are loaded and overridden in the following order: <br/>
 * <ol>
 * <li>default values from
 * org/openxma/dsl/generator/default-generator.properties located inside the
 * platform archive</li>
 * <li>/generator.properties from current classpath</li>
 * <li>file location as specified by the system property 'generator.properties'
 * (-Dgenerator.properties=file:/..)</li>
 * <li>{@link Properties} instance provided with the constructor of this class</li>
 * </ol>
 */
public class DefaultGeneratorContext implements GeneratorContext {
	private LayoutStrategy layoutStrategy;

	public DefaultGeneratorContext(Properties properties) {
		DslProperties.Holder.setProperties(properties);
		NamingStrategy namingStrategy = newInstanceOf(getProperty(DslConstants.DOMAIN_MODEL_NAMINGSTRATEGY_CLASS));
		NamingStrategy.Holder.setNamingStrategy(namingStrategy);
		ModelModifier modelModifier = newInstanceOf(getProperty(DslConstants.DOMAIN_MODEL_MODIFIER_CLASS));
		ModelModifier.Holder.setModelModifier(modelModifier);
	}

	public String getProperty(String key) {
		return DslProperties.Holder.getDslProperties().getProperty(key);
	}
	
	public String getProperty(String key, String defaultValue) {
		return DslProperties.Holder.getDslProperties().getProperty(key,defaultValue);
	}

	public ModelModifier getModelModifier() {
		return ModelModifier.Holder.getModelModifier();
	}

	public NamingStrategy getNamingStrategy() {
		return NamingStrategy.Holder.getNamingStrategy();
	}

	public LayoutStrategy getLayoutStrategy() {
		if (null == layoutStrategy) {
			this.layoutStrategy = newInstanceOf(getProperty(DslConstants.PRESENTATION_MODEL_LAYOUTSTRATEGY_CLASS));
		}
		return layoutStrategy;
	}

	private <T> T newInstanceOf(String className) {
		try {
			Class<T> clazz = loadClass(className);
			return clazz.newInstance();
		} catch (Exception exception) {
			throw new IllegalStateException(exception);
		}
	}

	@SuppressWarnings("unchecked")
	private <T> Class<T> loadClass(String className) throws ClassNotFoundException {
		return (Class<T>) ResourceLoaderFactory.createResourceLoader().loadClass(className);
	}

}
